define(['jquery', 'underscore', 'backbone', 'collections/tables', 'views/sidebar/table', 'events', 'context'],
    function ($, _, Backbone, Tables, TableView, Events, Context) {
        return Backbone.View.extend({
            emptyTemplate: Handlebars.compile('<li class="empty-text">Empty database</a>'),
            initialize: function () {
                this.tables = new Tables();
                this.listenTo(this.tables, 'add', this.add);
                _.bindAll(this, "activateFirstTable", "setActiveTable", "triggerTableChange", 
                    "tableFetchError", "resetActiveTable", "fetchTables", "add");

                // listen to global db:change event
                this.listenTo(Events, "db:change", this.fetchTables, this);

                // listen to global table:change event
                this.listenTo(Events, "table:change", this.setActiveTable, this);

                // listen to global queryMode:change event
                this.listenTo(Events, "queryMode:change", this.triggerTableChange);

                // listen to global table:navigate event
                this.listenTo(Events, 'table:navigate', this.navigateTable);

                // listen to global table:create event
                this.listenTo(Events, 'table:create', this.createTable, this);

                // listen to global table:changed event
                this.listenTo(Events, 'table:changed', this.updateTablesInContext, this);
                // already called in navigate table

                // listen to global table:deleted event
                this.listenTo(Events, 'table:deleted', this.tableDeleted);
            },
            updateTablesInContext: function () {
                // update context tables before navigation
                Context.tables = this.tables.map(function (table) { 
                    return table.get("title"); 
                });
                Events.trigger("tables:context-updated");
            },
            navigateTable: function (table) {
                // update tables before navigating, because 
                // other parts of datazenit expect the global 
                // tables list to be in sync
                // this was needed because of table:rename issue
                this.updateTablesInContext();

                this.$(".table-item").filter(function () {
                    return $(this).text().trim() === table;
                }).click();
            },
            createTable: function (model) {
                console.log("createTable", model, this.tables);
                this.tables.add(model);
                this.add(model);
            },
            tableDeleted: function (title) {
                if (this.activeTable && this.activeTable.get("title") === title) {
                    this.$(".table-meta").hide();
                }
            },
            setActiveTable: function (table, meta) {
                this.activeTable = table;
                this.activeMeta = meta;

                // hide other table meta
                // this prevents duplicate meta glitch on slow connections
                //this.$(".table-meta").hide();
                //this.$(".active").next(".table-meta").show();
            },
            fetchTables: function () {

                this.resetActiveTable();

                // start loading indicator
                this.$(".heading:first").text("Loading Tables...")

                if (Context.db) {
                    this.tables = new Tables();
                    this.$el.find("li:not(.heading)").remove();
                    this.tables.fetch({remove: false, success: this.activateFirstTable, error: this.tableFetchError});
                }
            },
            tableFetchError: function () {
                Events.trigger("error", "tables");
                // alert("Unable to load the list of tables.");
                this.$el.append(this.emptyTemplate());
            },
            triggerTableChange: function () {
                console.info("triggerTableChange", this.activeTable, this.activeMeta);
                // only trigger table change event, if we really have the data
                if (this.activeTable && this.activeMeta) {
                    Events.trigger("table:change", this.activeTable, this.activeMeta);
                }
            },
            resetActiveTable: function () {
                this.activeTable = null;
                this.activeMeta = null;
            },
            activateFirstTable: function (model, res, xhr) {

                // manually call add method on each model
                model.each(this.add);

                // stop loading indicator
                this.$(".heading:first").text("Tables")

                // store tables in context
                Context.tables = res;

                if (this.tables.length === 0) {
                    console.info("[!] database has no tables");
                    Backbone.history.syncURL();
                    this.$el.append(this.emptyTemplate());

                    // @todo is this really an error?
                    // Events.trigger("error", "tables");

                    return;
                }

                var $tableItem = this.$(".table-item").filter(function () {
                    return $(this).text().trim() === Context.table;
                });

                if (Context.table === null || $tableItem.length === 0) {
                    //console.log("activating first table in the list, because table from context not found:", Context.table);
                    //this.$("li>a:first").trigger("click");
                } else {
                    console.log("table found in context, activating it");
                    $tableItem.trigger("click");
                    // more elegant solution here @FIXME
                }

                // disable error triggering
                //var activeItemLength = this.$("li.active").length;
                //console.log("activated table item length", activeItemLength);
                //
                //if (!activeItemLength) {
                //    Events.trigger("error", "table", "Unable to find a table to activate.");
                //}

                Events.trigger("tables:fetched", model, res);

                // set tooltip for each table
                this.$(".builder").tooltip();
            },
            add: function (table) {

                // remove empty placeholder when adding a new table
                this.$(".empty-text").remove();

                var view = new TableView({ model: table });
                this.$el.append(view.render().el);
            }
        });
    });
